<?php

namespace Schaffen\Financial\Payment\Services;


use Schaffen\Financial\Payment\DTOs\GeneratePaymentInput;
use Schaffen\Financial\Payment\Exceptions\PaymentException;
use Schaffen\Financial\Payment\Models\AssociatePaymentInstallment;
use Schaffen\Financial\Payment\Models\AssociatePaymentPaymentMethod;

class ManipulatePayment
{
    public function getValue(GeneratePaymentInput $input){
        $x = round($input->getDesconto(), 2) -
            round($input->getJurosAtraso(), 2) -
            round($input->getMulta(), 2) -
            round($input->getJurosAdicionais(), 2) -
            round($input->getMoeda(), 2) -
            round($input->getAdicional(), 2);
        $y = round($input->getPagamento(), 2);
        return round($y + $x, 2);
    }

    public function validate(GeneratePaymentInput $input, $installment){
        if (!$installment) {
            throw new PaymentException("Não foi possível encontrar os dados da parcela {$input->getParCodigo()}. Verifique se a mesma não foi removida do sistema.");
        }
        if ($installment->par_status == 2) {
            throw new PaymentException("A parcela {$input->getParCodigo()}, de valor R$ {$installment->par_valor_original} encontra-se quitada.");
        }
        if(now()->endOfDay()->lte($input->getDataPagamento())){
            throw new PaymentException("A data de pagamento não pode ser maior que a data de hoje.");
        }
    }

    public function associateWithPaymentMethod($codPaymentMethod, $codPayment, $value){
        $temp = new AssociatePaymentPaymentMethod([
            "fpp_fk_fpg_codigo" => $codPaymentMethod,
            "fpp_fk_pag_codigo" => $codPayment,
            "fpp_valor" => $value
        ]);
        $temp->save();
        return $temp;
    }

    public function associateWithInstallment(array $values){
        $temp = new AssociatePaymentInstallment($values);
        $temp->save();
        return $temp;
    }
}